/**
 * Copyright (C) 2020 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * See about document.execCommand: https://developer.mozilla.org/en-US/docs/Web/API/Document/execCommand
 */


const {
    SlateEditor,
    SlateTransforms,
    SlateText
} = window.wangEditor
const {
    createEditor,
    createToolbar
} = window.wangEditor

/**
  exports.Boot = Boot;
  exports.DomEditor = dist$6.DomEditor;
  exports.SlateEditor = Editor_1;
  exports.SlateElement = Element_1;
  exports.SlateLocation = Location_1;
  exports.SlateNode = Node_1;
  exports.SlatePath = Path_1;
  exports.SlatePoint = Point_1;
  exports.SlateRange = Range_1;
  exports.SlateText = Text_1;
  exports.SlateTransforms = Transforms_1;
  exports.Toolbar = dist$6.Toolbar;
  exports.createEditor = createEditor;
  exports.createToolbar = createToolbar;
  exports.createUploader = dist$6.createUploader;
  exports["default"] = index;
  exports.genModalButtonElems = dist$6.genModalButtonElems;
  exports.genModalInputElems = dist$6.genModalInputElems;
  exports.genModalTextareaElems = dist$6.genModalTextareaElems;
  exports.i18nAddResources = dist$6.i18nAddResources;
  exports.i18nChangeLanguage = dist$6.i18nChangeLanguage;
  exports.i18nGetResources = dist$6.i18nGetResources;
  exports.t = dist$6.t;
  */

var RE = {};



RE.isInit = false;

RE.editorContainer = null
RE.editor = null;

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0
};


RE.setHtml = function (contents, focus) {
    if (!RE.isInit) {



        const editorConfig = {
            placeholder: '开始输入正文',
            onChange() {
                RE.queryBgColorNodes()
            }
        }

        const editor = createEditor({
            selector: '#editor-container',
            html: decodeURIComponent(contents.replace(/\+/g, '%20')),
            config: editorConfig,
            mode: 'simple', // or 'simple'
        })


        document.getElementById('editor-container').addEventListener('click', e => {
            console.log(e.target.id)
            if (e.target.id === 'editor-container') {
                editor.blur()
                editor.focus(true) // focus 到末尾
            }
        })

        RE.editorContainer = document.getElementById('editor-container');
        RE.editor = editor;
        RE.isInit = true

        RE.editor.on('change', function () {
            try {
                //        console.log('change')
                var editStyle = SlateEditor.marks(RE.editor)
                var items = [];
                if (editStyle.bold != null && editStyle.bold) {
                    items.push('bold');
                }
                if (editStyle.italic != null && editStyle.italic) {
                    items.push('italic');
                }
                if (editStyle.sub != null && editStyle.sub) {
                    items.push('subscript');
                }
                if (editStyle.sup != null && editStyle.sup) {
                    items.push('superscript');
                }
                if (editStyle.underline != null && editStyle.underline) {
                    items.push('underline');
                }
                if (editStyle.through != null && editStyle.through) {
                    items.push('through');
                }

                const fragment = RE.editor.getFragment()
                //        console.log(fragment[0])
                if (fragment != null) {
                    var type = fragment[0].type
                    if (type == "header1") {
                        items.push('H1');
                    }
                    if (type == "header2") {
                        items.push('H2');
                    }
                    if (type == "header3") {
                        items.push('H3');
                    }
                    if (type == "header4") {
                        items.push('H4');
                    }
                    if (type == "header5") {
                        items.push('H5');
                    }
                    if (type == "header6") {
                        items.push('H6');
                    }

                    if (type == "list-item") {
                        if (fragment[0].ordered == true) {
                            items.push('orderedList');
                        } else {
                            items.push('unorderedList');
                        }
                    }

                    var textAlign = fragment[0].textAlign
                    if (textAlign == "center") {
                        items.push('justifyCenter');
                    }
                    if (textAlign == "left") {
                        items.push('justifyLeft');
                    }
                    if (textAlign == "right") {
                        items.push('justifyRight');
                    }
                }
                console.log(items.join(','))
                window.location.href = "re-state://" + encodeURI(items.join(','));
                window.location.href = "re-callback://" + encodeURIComponent(RE.getHtml());
            } catch (e) {
                window.location.href = "re-callback://" + encodeURIComponent(RE.getHtml());
                console.log(e)
                return
            }

        })
        // 搜索带背景颜色的节点
        RE.queryBgColorNodes = function () {
            try {
                var key = '#w-e-textarea-1';
                var parentNode = document.querySelector(key);
                if (!parentNode) {
                    return
                }
                let result = [];
                const resultObj = {};
                const Nodes = Array.from(parentNode.querySelectorAll(`*[style*="color: "]`));
                Nodes.forEach((item) => {
                    if (item.style.color === 'transparent') {
                        return;
                    }
                    const key = item.style.color;
                    const rect = item.getBoundingClientRect();
                    if (!resultObj.hasOwnProperty(key)) {
                        resultObj[key] = {
                            positions: rect,
                            nodes: [item]
                        }
                    } else {
                        resultObj[key].nodes.push(item);
                    }
                })
                result = Object.keys(resultObj).map(key => {
                    const item = resultObj[key];
                    item.comment_id = key;
                    return item
                })
                console.log(result)

                var nodeTops = [];
                for (let i = 0; i < result.length; ++i) {
                    var nodeTop = {
                        key: result[i].comment_id,
                        top: result[i].positions.top,
                    };
                    var nodes = result[i].nodes;
                    for (let j = 0; j < nodes.length; ++j) {
                        //                console.log(nodes[j])
                        nodes[j].onclick = function () {
                            window.hwJs.onShowReview(i, commentNodes[i].innerText);
                        };
                    }
                    nodeTops.push(nodeTop);
                }
                console.log(JSON.stringify(nodeTops))
                window.WangEditorJs.onQueryBgColorNodes(JSON.stringify(nodeTops));
            } catch (e) {
                console.log(e);
                return
            }
        }
        RE.editor.on('scroll', () => {
            RE.queryBgColorNodes()
        })
    }


    RE.editor.setHtml(decodeURIComponent(contents.replace(/\+/g, '%20')));
    if (focus) {
        RE.editor.focus(true)
    }
    RE.queryBgColorNodes()
}

RE.getHtml = function () {
    return RE.editor.getHtml();
}

RE.startSelect = function () {
    var selection = window.getSelection();
    selection.modify('extend', 'forward', "paragraph");
}


RE.disable = function () {
    return RE.editor.disable();
}

RE.enable = function () {
    return RE.editor.enable();
}


RE.getText = function () {
    return RE.editor.getText();
}

RE.setBaseTextColor = function (color) {}

RE.setBaseFontSize = function (size) {}

RE.setPadding = function (left, top, right, bottom) {
    RE.editorContainer.style.paddingLeft = left;
    RE.editorContainer.style.paddingTop = top;
    RE.editorContainer.style.paddingRight = right;
    RE.editorContainer.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function (color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function (image) {
    RE.editorContainer.style.backgroundImage = image;
}

RE.setWidth = function (size) {
    RE.editorContainer.style.minWidth = size;
}

RE.setHeight = function (size) {
    RE.editorContainer.style.height = size;
    RE.queryBgColorNodes()
}

RE.setHeight = function (size, focus) {
    if (focus) {
        RE.editor.blur()
    }
    RE.editorContainer.style.height = size;
    if (focus) {
        RE.editor.restoreSelection()
    }
    RE.queryBgColorNodes()
}

RE.reFocus = function () {
    if (!RE.editor.isFocused()) {
        RE.editor.restoreSelection()
    }
}

RE.setTextAlign = function (align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function (align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function (placeholder) {
    editorConfig.placeholder = placeholder;
}

RE.setInputEnabled = function (inputEnabled) {
    if (inputEnabled) {
        RE.editor.enable()
    } else {
        RE.editor.disable()
    }
}

RE.undo = function () {
    RE.editor.undo()
}

RE.redo = function () {
    RE.editor.redo()
}

RE.setBold = function () {
    var isTrue = SlateEditor.marks(RE.editor).bold
    if (isTrue) {
        RE.editor.removeMark('bold')
    } else {
        RE.editor.addMark('bold', true)
    }
}

RE.setItalic = function () {
    var isTrue = SlateEditor.marks(RE.editor).italic
    if (isTrue) {
        RE.editor.removeMark('italic')
    } else {
        RE.editor.addMark('italic', true)
    }
}

RE.setSubscript = function () {
    var isTrue = SlateEditor.marks(RE.editor).sub
    if (isTrue) {
        RE.editor.removeMark('sub')
    } else {
        RE.editor.addMark('sub', true)
    }
}

RE.setSuperscript = function () {
    var isTrue = SlateEditor.marks(RE.editor).sup
    if (isTrue) {
        RE.editor.removeMark('sup')
    } else {
        RE.editor.addMark('sup', true)
    }
}

RE.setStrikeThrough = function () {
    var isTrue = SlateEditor.marks(RE.editor).sup
    if (isTrue) {
        RE.editor.removeMark('through')
    } else {
        RE.editor.addMark('through', true)
    }
}

RE.setUnderline = function () {
    var isTrue = SlateEditor.marks(RE.editor).underline
    if (isTrue) {
        RE.editor.removeMark('underline')
    } else {
        RE.editor.addMark('underline', true)
    }
}

RE.setBullets = function () {
    SlateTransforms.setNodes(RE.editor, {
        type: 'list-item',
        ordered: false, // 有序 true/无序false
        indent: undefined,
    })
}

RE.setNumbers = function () {
    SlateTransforms.setNodes(RE.editor, {
        type: 'list-item',
        ordered: true, // 有序 true/无序false
        indent: undefined,
    })
}

RE.setTextColor = function (color) {
    RE.editor.addMark('color', color)
}

RE.setTextBackgroundColor = function (color) {
    RE.editor.addMark('bgColor', color)
}

RE.setFontSize = function (fontSize) {
    RE.editor.addMark('fontSize', fontSize)
}

RE.setHeading = function (heading) {
    SlateTransforms.setNodes(RE.editor, {
        type: 'header' + heading,
    })
}

RE.setParagraph = function () {
    SlateTransforms.setNodes(RE.editor, {
        type: 'paragraph',
    })
}

RE.setIndent = function () {
    SlateTransforms.setNodes(RE.editor, {
        indent: '2em'
    })
}

RE.setOutdent = function () {
    SlateTransforms.setNodes(RE.editor, {
        indent: '0em'
    })
}

RE.setJustifyLeft = function () {
    SlateTransforms.setNodes(RE.editor, {
        textAlign: 'left',
    })
}

RE.setJustifyCenter = function () {
    SlateTransforms.setNodes(RE.editor, {
        textAlign: 'center',
    })
}

RE.setJustifyRight = function () {
    SlateTransforms.setNodes(RE.editor, {
        textAlign: 'right',
    })
}

RE.setBlockquote = function () {
    document.execCommand('formatBlock', false, '<blockquote>');
}

RE.insertImage = function (url, alt) {
    RE.editor.restoreSelection()
    var image = {
        type: 'image',
        src: url,
        style: {
            width: '100%'
        },
        children: [{
            text: alt
        }]
    }
    RE.editor.insertNode(image)
}

RE.insertImageW = function (url, alt, width) {
    RE.editor.restoreSelection()
    var image = {
        type: 'image',
        src: url,
        style: {
            width: width
        },
        children: [{
            text: alt
        }]
    }
    RE.editor.insertNode(image)
}

RE.insertImageWH = function (url, alt, width, height) {
    RE.editor.restoreSelection()
    var image = {
        type: 'image',
        src: url,
        style: {
            width: width,
            height: height
        },
        children: [{
            text: alt
        }]
    }
    RE.editor.insertNode(image)
}

RE.insertVideo = function (url, alt) {
    RE.editor.restoreSelection()
    var image = {
        type: 'video',
        src: url,
        style: {
            width: '100%',
        },
        children: [{
            text: ''
        }]
    }
    RE.editor.insertNode(image)
}

RE.insertVideoW = function (url, width) {
    RE.editor.restoreSelection()
    var image = {
        type: 'video',
        src: url,
        style: {
            width: width
        },
        children: [{
            text: ''
        }]
    }
    RE.editor.insertNode(image)
}

RE.insertVideoWH = function (url, width, height) {
    RE.editor.restoreSelection()
    var image = {
        type: 'video',
        src: url,
        style: {
            width: width,
            height: height
        },
        children: [{
            text: ''
        }]
    }
    RE.editor.insertNode(image)
}

RE.insertAudio = function (url, alt) {
    RE.editor.restoreSelection()
    var html = '<audio src="' + url + '" controls></audio><br>';
    RE.insertHTML(html);
}

RE.insertYoutubeVideo = function (url) {
    var html = '<iframe width="100%" height="100%" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoW = function (url, width) {
    var html = '<iframe width="' + width + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoWH = function (url, width, height) {
    var html = '<iframe width="' + width + '" height="' + height + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertHTML = function (html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
}

RE.insertLink = function (url, title) {
    var image = {
        type: 'link',
        url: url,
        target: title,
        children: [{
            text: "title"
        }]
    }
    RE.editor.insertNode(image)
}

RE.setTodo = function (text) {
    //    var html = '<input type="checkbox" name="' + text + '" value="' + text + '"/> &nbsp;';
    //    document.execCommand('insertHTML', false, html);
}


RE.removeTypeFormat = function (type) {
    RE.editor.selectAll()
    RE.editor.removeMark(type)
}



RE.focus = function () {
    RE.editor.focus()
}

RE.blurFocus = function () {
    RE.editor.blur();
}

RE.removeFormat = function () {
    document.execCommand('removeFormat', false, null);
}





RE.highlight = function (position) {
    try {
        var key = '#w-e-textarea-1'
        var parentNode = document.querySelector(key)
        if (!parentNode) {
            return
        }
        let result = [];
        const resultObj = {};
        const Nodes = Array.from(parentNode.querySelectorAll(`*[style*="color: "]`));
        Nodes.forEach((item) => {
            if (item.style.color === 'transparent') {
                return
            }
            const key = item.style.color;
            const rect = item.getBoundingClientRect()
            if (!resultObj.hasOwnProperty(key)) {
                resultObj[key] = {
                    positions: rect,
                    nodes: [item]
                }
            } else {
                resultObj[key].nodes.push(item)
            }
        })

        result = Object.keys(resultObj).map(key => {
            const item = resultObj[key];
            item.comment_id = key;
            return item
        });

        for (let i = 0; i < result.length; ++i) {
            var nodes = result[i].nodes;
            if (i == position) {
                for (let j = 0; j < nodes.length; ++j) {
                    if (j == 0) {
                        nodes[j].id = 'node' + i + '' + j;
                    }
                    nodes[j].style.backgroundColor = 'rgba(192,74,74, 0.25)';
                }
            } else {
                for (let j = 0; j < nodes.length; ++j) {
                    if (j == 0) {
                        nodes[j].id = 'node' + i + '' + j;
                    }
                    nodes[j].style.backgroundColor = 'rgba(255,159,2, 0.25)';
                }
            }
        }
        document.getElementById(result[position].nodes[0].id).scrollIntoView();
    } catch (e) {
        console.log(e)
        return
    }
}
